//                               -*- Mode: c++ -*- 
// SourceUnit.cc --- Source Unit sub-hierarchy
// Author          : Nicolas Anquetil
// Created On      : Tue Nov 26 23:12:22 1996
//

#include "SourceUnit.hh"


// **************************************************
//					  SourceUnit
// **************************************************


void SourceUnit::addReference( ReferenceExistence *usedEntity, bool isComment)
// --------------------------------------------------
{
  if (isComment)
	addInSet( refersToCommentTerm(), (CommentTermExistence*)usedEntity);
  else
	addInSet( refersToData(), (DataUseExistence*)usedEntity);
}  /* SourceUnit::addReference() */


VoidIterator<SoftwareObject*> *SourceUnit::defines(class_t type)
//--------------------------------------------------
// may define constant, variable, type or routine.
{
  VoidIterator<SoftwareObject*> *iter;

  if ( (isCdbSuperClass(type, _routinesource)) ||
       (isCdbSuperClass( _routinesource, type)) )  // sy
	iter = new TypeFilterIterator<SourceWithinFile*,SoftwareObject*>(
	         new LIterator<SourceWithinFile*>( contains()),
			 type);
  else
	iter = new VoidIterator<SoftwareObject*>();

  if ( isCdbSuperClass(type, _definition) ||
       isCdbSuperClass(_definition, type) )
    iter = new SortedUnionIterator<SoftwareObject*>(
	     iter,
             new TypeFilterIterator<StandaloneDefinition*,SoftwareObject*>(
	           new LIterator<StandaloneDefinition*>( defines()),
		   type),
	     SO_loose_cmp, SOmerge);

 return(iter);
}  /* SourceUnit::defines() */


VoidIterator<SoftwareObject*> *SourceUnit::refersTo(class_t type)
//--------------------------------------------------
// may refer to constant, variable, type or comment.
{
  VoidIterator<SoftwareObject*> *iter;

  if (isCdbSuperClass(type, _specialcodeexistence) ||
      isCdbSuperClass(_specialcodeexistence, type) )
    iter = new TypeFilterIterator<CommentTermExistence*,SoftwareObject*>(
	     new LIterator<CommentTermExistence*>(
               refersToCommentTerm()),
	     type);
  else
	iter = new VoidIterator<SoftwareObject*>();

    iter = new SortedUnionIterator<SoftwareObject*>(
             iter,
             new TypeFilterIterator<DataUseExistence*,SoftwareObject*>(
	           new LIterator<DataUseExistence*>( refersToData()),
			   type),
			 SO_loose_cmp, SOmerge);

    iter = new SortedUnionIterator<SoftwareObject*>(
             iter,
             new TypeFilterIterator<RoutineCallExistence*,SoftwareObject*>(
	           new LIterator<RoutineCallExistence*>( calls()),
			   type),
			 SO_loose_cmp, SOmerge);

  return(iter);
}  /* SourceUnit::refersTo() */


# ifdef DEBUG
void SourceUnit::hello()
// --------------------------------------------------
{
  printf("SourceUnit\nis a ");
  SoftwareObject::hello();
  printf("\tcontains %d source unit\n", contains()->size());
  printf("\trefers to %d data/type/const\n", refersToData()->size());
  printf("\trefers to %d comment terms\n", refersToCommentTerm()->size());
  printf("\tdefines %d entities\n", defines()->size());
}  /* SourceUnit::hello() */
# endif


// **************************************************
//					  SourceFile
// **************************************************


SourceFile::SourceFile(char *fName,
// --------------------------------------------------
    int   vers,
    char *fPath,
    int   dChange,
    int   fSize)
{
  Name *n;

# ifdef DEBUG
  printf("SourceFile::SourceFile(%s)\n",fName);
# endif
  n = the_DB->getName(fName);
  _fileName    = n;
  n->addNamedFile( this);
  _version     = vers;

  // Allocate a copy of the path
  _path = (char *)allocFileMem(strlen(fPath)+1);
  strcpy(_path, fPath);
  _dateChanged = dChange;
  _size        = fSize;

  the_DB->allFiles()->add(this);

  the_DB->incCount( classType());
}  /* SourceFile::SourceFile() */


VoidIterator<SoftwareObject*> *SourceFile::refersTo(class_t type)
//--------------------------------------------------
// May refer to "same as SourceUnit" or file
{
  VoidIterator<SoftwareObject*> *iter;

  if (isCdbSuperClass( type, _fileinclusionexistence) ||
      isCdbSuperClass( _fileinclusionexistence, type) )
    iter = new TypeFilterIterator<FileInclusionExistence*,SoftwareObject*>(
             new LIterator<FileInclusionExistence*>( includes()),
	     type);
  else
	iter = new VoidIterator<SoftwareObject*>();

  return( new SortedUnionIterator<SoftwareObject*>(
            iter, SourceUnit::refersTo(type), SO_loose_cmp, SOmerge) );

}  /* SourceFile::refersTo() */


# ifdef DEBUG
void SourceFile::hello()
// --------------------------------------------------
{
  printf("SourceFile (%s/%s)\nis a ", pathName(), baseName()->string());
  SourceUnit::hello();
  printf("\tsize: %d\n", size());
  printf("\tversion: %d\n", version());
  printf("\tdate changed: %d\n", dateChanged());
  printf("\tincludes %d files\n", includes()->size());
}  /* SourceFile::hello() */
# endif


// **************************************************
//					SourceWithinFile
// **************************************************

SourceWithinFile::SourceWithinFile(SourceUnit *containerToBe,
// --------------------------------------------------
								   int startPos,
								   int endPos)
{
# ifdef DEBUG
  printf("SourceWithinFile::SourceWithinFile\n");
# endif
  _containingSource = containerToBe;
  _startChar = startPos;
  _endChar   = endPos;

  if (containerToBe != NULL)
    containerToBe->addContainedSource( this);
  the_DB->incCount(classType());
}  /* SourceWithinFile::SourceWithinFile() */


# ifdef DEBUG
void SourceWithinFile::hello()
// --------------------------------------------------
{
  printf("SourceWithinFile\nis a ");
  SourceUnit::hello();
  printf("\tPos, begin:%d, end:%d\n", startChar(), endChar());
  printf("containing source --------------------\n");
  containingSource()->hello();
  printf("-------------------- containing source\n");
}  /* SourceWithinFile::hello() */
# endif


// **************************************************
//					RoutineSource
// **************************************************


RoutineSource::RoutineSource(SourceUnit *containerToBe,
// --------------------------------------------------
							 int   startPos,
							 int   endPos,
							 char *rtnName,
							 bool  entry_p,
							 TypeUseExistence *returns)
	: SourceWithinFile(containerToBe,startPos,endPos)
{
  Name *n;

# ifdef DEBUG
  printf("RoutineSource::RoutineSource(%s)\n",rtnName);
# endif
  n = the_DB->getName(rtnName);
  _routineName = n;
  n->addNamedRoutine( this);

  _isEntry    = entry_p;
  _returnType = returns;

  if (returns != NULL)
    returns->addUseReturnType(this);
  // If containerToBe and returns are not NULL, some other function has
  // to create the links `foundInSource' in the typeUseExistence and
  // `refersToData' in the container

  the_DB->allRoutines()->add(this);

  the_DB->incCount( classType());
}  /* RoutineSource::RoutineSource() */


List<DatumDef *> *RoutineSource::formalArgs()
// --------------------------------------------------
{
  LIterator<StandaloneDefinition *> *iter;
  List<DatumDef *> *retn;
  StandaloneDefinition *obj;
  
  retn = new List<DatumDef *>;
  iter = new LIterator<StandaloneDefinition *>(defines());

  while (iter->eoi()) {
	obj = iter->next();
	if ( obj->is_a(_datumdef) && ((DatumDef *)obj)->isFormalArgument() )
	  retn->add((DatumDef *)obj);
  }

  delete(iter);

  return( retn);
}  /* RoutineSource::formalArgs() */


VoidIterator<SoftwareObject*> *RoutineSource::definedBy(class_t type)
//--------------------------------------------------
// May be defined by file or routine
{
  if (containingSource() == NULL)
	return(new VoidIterator<SoftwareObject*>());
  else
	return( new TypeFilterIterator<SourceUnit*,SoftwareObject*>(
              new SingletonIterator<SourceUnit*>( containingSource()),
			  type) );
}  /* RoutineSource::definedBy() */


VoidIterator<SoftwareObject*> *RoutineSource::refersTo(class_t type)
//--------------------------------------------------
// May refer to "same as SourceUnit" or routine
{
  VoidIterator<SoftwareObject*> *iter;

  if (isCdbSuperClass(type, _routinecallexistence) ||
      isCdbSuperClass(_routinecallexistence, type) )
    iter = new TypeFilterIterator<RoutineCallExistence*,SoftwareObject*>(
             new LIterator<RoutineCallExistence*>( calls()),
	     type);
  else
   	iter = new VoidIterator<SoftwareObject*>();

    return( new SortedUnionIterator<SoftwareObject*>(
	      iter, SourceUnit::refersTo(type), SO_loose_cmp, SOmerge) );
}  /* RoutineSource::refersTo() */


# ifdef DEBUG
void RoutineSource::hello()
// --------------------------------------------------
{
  printf("RoutineSource (%s)\nis a ", routineName()->string());
  SourceWithinFile::hello();
  printf("\tentry:%c, fct:%c\n", isEntry()?'Y':'N', isFunction()?'Y':'N');
  printf("\tcalls %d routines\n", calls()->size());
  printf("\thas %d args\n", formalArgs()->size());
  if (isFunction()) {
	printf("return type --------------------\n");
	returnType()->hello();
	printf("-------------------- return type\n");
  }
}  /* RoutineSource::hello() */
# endif
