/*                              -*- Mode: C++ -*- 
 * cdbClasses.c --- utilities to manipulate the softwareobject
 *                   classes hierarchie
 * Author          : Nicolas Anquetil
 * Created On      : Mon Aug 18 12:14:52 1997
 */ 

# include "cdbClasses.h"


bool
isCdbVirtualClass( class_t c)
/**************************************************
 */
{
  return( (c == _file) || (c == _routine) || (c == _data) || (c == _type) );
}


bool
isCdbAbstractClass( class_t c)
/**************************************************
 */
{
  return( (c == _softwareobject)     || (c == _sourceunit)  ||
		  (c == _sourcewithinfile)   || (c == _sourceblock) ||
		  (c == _referenceexistence) || (c == _definition)  || 
		  (c == _typeddefinition)    || (c == _standalonedefinition) );
}


bool
isCdbDefinitionClass( class_t c)
/**************************************************
 */
{
  return( (c == _sourcefile)        || (c == _routinesource)    ||
		  (c == _field)             || (c == _typedef)          || 
		  (c == _datumdef)          || (c == _enumerationconst) || 
		  (c == _enumeratedtypedef) || (c == _recordtypedef) );
}


bool
isCdbReferenceClass( class_t c)
/**************************************************
 */
{
  return( (c == _commenttermexistence)   || (c == _specialcodeexistence) ||
		  (c == _fileinclusionexistence) || (c == _datauseexistence)     || 
		  (c == _manifestconstexistence) || (c == _typeuseexistence)     || 
		  (c == _routinecallexistence) );
}


bool
isCdbSuperClass( class_t superT, class_t subT)
/**************************************************
 * is superT a super class of subT ?
 * Horrible object programming, we have to hard code the inheritance
 * hierarchy !!
 */
{
  if (superT == subT)
	return(true);
  else if (isCdbVirtualClass(superT))
	return(virtualizeclass(subT) == superT);
  else
    switch (subT) {
      /* because superT != subT, it cannot possibly be superclass of SoftObj */
    case _softwareobject:         return(false);
    case _sourceunit:             return(isCdbSuperClass(superT,_softwareobject));
    case _sourcefile:             return(isCdbSuperClass(superT,_sourceunit));
    case _sourcewithinfile:       return(isCdbSuperClass(superT,_sourceunit));
    case _routinesource:          return(isCdbSuperClass(superT,_sourcewithinfile));
    case _sourceblock:            return(isCdbSuperClass(superT,_sourcewithinfile));

    case _referenceexistence:     return(isCdbSuperClass(superT,_softwareobject));
    case _commenttermexistence:   return(isCdbSuperClass(superT,_referenceexistence));
    case _specialcodeexistence:   return(isCdbSuperClass(superT,_commenttermexistence));
    case _fileinclusionexistence: return(isCdbSuperClass(superT,_referenceexistence));
    case _datauseexistence:       return(isCdbSuperClass(superT,_referenceexistence));
    case _manifestconstexistence: return(isCdbSuperClass(superT,_referenceexistence));
    case _typeuseexistence:       return(isCdbSuperClass(superT,_referenceexistence));
    case _routinecallexistence:   return(isCdbSuperClass(superT, _referenceexistence));

    case _definition:             return(isCdbSuperClass(superT,_softwareobject));
    case _typeddefinition:        return(isCdbSuperClass(superT,_definition));
    case _standalonedefinition:   return(isCdbSuperClass(superT,_typeddefinition));
    case _field:                  return(isCdbSuperClass(superT,_typeddefinition));
    case _typedef:                return(isCdbSuperClass(superT,_standalonedefinition));
    case _datumdef:               return(isCdbSuperClass(superT,_standalonedefinition));
    case _enumerationconst:       return(isCdbSuperClass(superT,_definition));
    case _enumeratedtypedef:      return(isCdbSuperClass(superT,_typedef));
    case _recordtypedef:          return(isCdbSuperClass(superT,_typedef));
    default:	                    return(false);
    }
}


char *
cdbClass2str(class_t clss)
/**************************************************
 * returns a string given an enumeration constant
 */
{
  switch(clss) {
  case _file:                   return("file");
  case _routine:                return("routine");
  case _data:                   return("data");
  case _type:                   return("type");

  case _commenttermexistence:   return("CommentTermExistence");
  case _datauseexistence:       return("DataUseExistence");
  case _datumdef:               return("DatumDef");
  case _enumeratedtypedef:      return("EnumeratedTypeDef");
  case _enumerationconst:       return("EnumerationConst");
  case _field:                  return("Field");
  case _fileinclusionexistence: return("FileInclusionExistence");
  case _manifestconstexistence: return("ManifestConstExistence");
  case _recordtypedef:          return("RecordTypeDef");
  case _routinecallexistence:   return("RoutineCallExistence");
  case _routinesource:          return("RoutineSource");
  case _sourcefile:             return("SourceFile");
  case _specialcodeexistence:   return("SpecialCodeExistence");
  case _typedef:                return("TypeDef");
  case _typeuseexistence:       return("TypeUseExistence");

  case _definition:             return("Definition");
  case _referenceexistence:     return("ReferenceExistence");
  case _softwareobject:         return("SoftwareObject");

  default:                      return("** warning: unknown type");
  }
}


char *
cdbClass2abbrev(class_t clss)
/**************************************************
 * returns an abbreviated version of the class name
 */
{
  switch(clss) {
  case _file:                   return("fil");
  case _routine:                return("rou");
  case _data:                   return("dat");
  case _type:                   return("typ");

  case _commenttermexistence:   return("cte");
  case _datauseexistence:       return("due");
  case _datumdef:               return("dd");
  case _enumeratedtypedef:      return("etd");
  case _enumerationconst:       return("ec");
  case _field:                  return("fld");
  case _fileinclusionexistence: return("fie");
  case _manifestconstexistence: return("mce");
  case _recordtypedef:          return("rtd");
  case _routinecallexistence:   return("rce");
  case _routinesource:          return("rs");
  case _sourcefile:             return("sf");
  case _specialcodeexistence:   return("sce");
  case _typedef:                return("td");
  case _typeuseexistence:       return("tue");

  case _definition:             return("de");
  case _referenceexistence:     return("re");
  case _softwareobject:         return("so");

  default:                      return("** warning: unknown type");
  }
}


class_t
str2cdbClass(char*str)
/**************************************************
 * returns a string given an enumeration constant
 */
{
  class_t clss;

  if (strcmp(str,"CommentTermExistence") == 0)
    clss = _commenttermexistence;
  else if (strcmp(str,"DataUseExistence") == 0)
    clss = _datauseexistence;
  else if (strcmp(str,"DatumDef") == 0)
    clss = _datumdef;
  else if (strcmp(str,"EnumeratedTypeDef") == 0)
    clss = _enumeratedtypedef;
  else if (strcmp(str,"EnumerationConst") == 0)
    clss = _enumerationconst;
  else if (strcmp(str,"Field") == 0)
    clss = _field;
  else if (strcmp(str,"FileInclusionExistence") == 0)
    clss = _fileinclusionexistence;
  else if (strcmp(str,"ManifestConstExistence") == 0)
    clss = _manifestconstexistence;
  else if (strcmp(str,"RecordTypeDef") == 0)
    clss = _recordtypedef;
  else if (strcmp(str,"RoutineCallExistence") == 0)
    clss = _routinecallexistence;
  else if (strcmp(str,"RoutineSource") == 0)
    clss = _routinesource;
  else if (strcmp(str,"SourceFile") == 0)
    clss = _sourcefile;
  else if (strcmp(str,"SpecialCodeExistence") == 0)
    clss = _specialcodeexistence;
  else if (strcmp(str,"TypeDef") == 0)
    clss = _typedef;
  else if (strcmp(str,"TypeUseExistence") == 0)
    clss = _typeuseexistence;
  else
	clss = _nullclass;

  return(clss);
}

class_t
virtualizeclass( class_t c)
/**************************************************
 */
{
  if      (isCdbVirtualClass(c))
	return(c);
  else if (isCdbAbstractClass(c))
	return( _nullclass);
  else
	switch (c) {
	case _sourcefile:
	case _fileinclusionexistence:
	  return( _file);
	  break;
	  
	case _routinesource:
	case _routinecallexistence:
	  return( _routine);
	  break;
	  
	case _datumdef:
	case _field:
	case _enumerationconst:
	case _datauseexistence:
	  return( _data);
	  break;
	  
	case _typedef:
	case _enumeratedtypedef:
	case _recordtypedef:
	case _typeuseexistence:
	  return( _type);
	  break;

	default:     /* e.g. _manifestconstexistence */
	  return( _nullclass);
	}
}


bool
compatibleclass( class_t c1, class_t c2)
/**************************************************
 * Comparison of 2 classes from the query language point of view
 * (i.e. virtual classes are the "superclasses" of actual classes)
 */
{
  return( virtualizeclass(c1) == virtualizeclass(c2) );
}

/**************************************************
 * Local Variables: 
 * tab-width: 4
 * End:
 */
