//                              -*- Mode: C++ -*- 
// newDB.cc --- Top of the new DataBase model hierarchy
// Author          : Nicolas Anquetil
// Created On      : Thu Nov 28 12:11:39 1996
// 

# include "utils.hh"

// These three are needed for the Name class' methods
// newDB.hh is loaded through these files
// Kind of ugly, but "needed" because they all include one each other.
# include "SourceUnit.hh"
# include "Definition.hh"
# include "ReferenceExistence.hh"

// **********************************************************************
//				   G L O B A L   V A R I A B L E S
// **********************************************************************

Database *the_DB;


// **************************************************
//					  Name
// **************************************************


VoidIterator<SoftwareObject*> *Name::getObjectList(
// --------------------------------------------------
// Given the type of an object we want, this function returns an
// iterator listing all the possible object
// Assumes the lists are sorted
    class_t filteringClass)
{
  return( new SortedUnionIterator<SoftwareObject*>(
		     new SortedUnionIterator<SoftwareObject*>(
               new SortedUnionIterator<SoftwareObject*>(
                 new TypeFilterIterator<SourceFile*,SoftwareObject*>(
                   new LIterator<SourceFile*>( namedFiles()),
				   filteringClass),
                 new TypeFilterIterator<RoutineSource*,SoftwareObject*>(
                   new LIterator<RoutineSource*>( namedRoutines()),
				   filteringClass),SO_strict_cmp, SOmerge ),
			   new TypeFilterIterator<Definition*,SoftwareObject*>(
                 new LIterator<Definition*>( namedDefinitions()),
				 filteringClass),SO_strict_cmp, SOmerge ),
			 new TypeFilterIterator<ReferenceExistence*,SoftwareObject*>(
               new LIterator<ReferenceExistence*>( namedReferences()),
			   filteringClass),SO_strict_cmp, SOmerge ) );
}  /* Name::getObjectList() */


VoidIterator<SoftwareObject*> *Name::possibleInstancesOfClass(
// --------------------------------------------------
// Given the type of an object we want, this function returns the list
// where we might find this object
// Doesn't assume the lists are sorted
    class_t classType)
{
  switch (classType) {
  case (_softwareobject) :
	// bad because hard encode the class hierarchy
	return( new AppendIterator<SoftwareObject*>(
			  new AppendIterator<SoftwareObject*>(
			    new AppendIterator<SoftwareObject*>(
                  possibleInstancesOfClass(_sourcefile),
				  possibleInstancesOfClass(_sourceunit)),
				possibleInstancesOfClass(_definition) ),
			  possibleInstancesOfClass(_referenceexistence)) );
	break;

  case (_sourceunit) :
	return( new AppendIterator<SoftwareObject*>(
			  possibleInstancesOfClass(_sourcefile),
			  possibleInstancesOfClass(_routinesource) ));
	break;

  case (_sourcefile) :
	return( new CastIterator<SourceFile*,SoftwareObject*>(
              new LIterator<SourceFile*>( namedFiles())));
	break;

  case (_sourcewithinfile) :
  case (_routinesource) :
	return( new CastIterator<RoutineSource*,SoftwareObject*>(
              new LIterator<RoutineSource*>( namedRoutines())));
	break;

  case (_referenceexistence) :
  case (_commenttermexistence) :
  case (_specialcodeexistence) :
  case (_fileinclusionexistence) :
  case (_datauseexistence) :
  case (_manifestconstexistence) :
  case (_typeuseexistence) :
  case (_routinecallexistence) :
	return( new CastIterator<ReferenceExistence*,SoftwareObject*>(
              new LIterator<ReferenceExistence*>( namedReferences())));
	break;

  case (_definition) :
  case (_typeddefinition) :
  case (_standalonedefinition) :
  case (_field) :
  case (_typedef) :
  case (_datumdef) :
  case (_enumerationconst) :
  case (_enumeratedtypedef) :
  case (_recordtypedef) :
	return( new CastIterator<Definition*,SoftwareObject*>(
              new LIterator<Definition*>( namedDefinitions())));
	break;

  default :
	return( new VoidIterator<SoftwareObject*>());
  }
}  /* Name::possibleInstancesOfClass() */


void Name::sortLists() 
//--------------------------------------------------
{
  sortTheList((List<SoftwareObject*>*)&_namedDefinitions);
  sortTheList((List<SoftwareObject*>*)&_namedFiles);
  sortTheList((List<SoftwareObject*>*)&_namedRoutines);
  sortTheList((List<SoftwareObject*>*)&_namedReferences);
}


// **************************************************
//					SoftwareObject
// **************************************************

SoftwareObject::SoftwareObject()
// --------------------------------------------------
{
# ifdef DEBUG
  printf("SoftwareObject::SoftwareObject()\n");
# endif
  the_DB->allObjects()->add(this);
  _objectKey = the_DB->allObjects()->size() - 1;

  the_DB->incCount( classType());
}  /* SoftwareObject::SoftwareObject() */


bool
SoftwareObject::is_a(class_t aType)
// --------------------------------------------------
{
  if (isCdbVirtualClass(aType))
	return( compatibleclass(aType,classType()));
  else
	return( (aType == _softwareobject));
}  /* SoftwareObject::is_a() */


# ifdef DEBUG
void
SoftwareObject::hello()
// --------------------------------------------------
{
  printf("SoftwareObject #(%d)\n", objectKey());
}  /* SoftwareObject::hello() */
# endif


// **************************************************
//                     Database
// **************************************************


Database::Database(char *vers)
//--------------------------------------------------
{
  int i;
# ifdef DEBUG
  printf("Database::Database()\n");
# endif
  _allNames = createHashTableSpecialMem(1000, dbKeyFunc, 0, allocFileMem,
										freeFileMem);

  for (i=0; i < MAX_CLASSES; i++)
	_counts[i] = 0;

  // The string containing the version must be in the DB, therefore
  // use "new" to allocate it in the "filemem" portion of the memory
  _version = new char[strlen(vers)+1];
  strcpy(_version,vers);   // and then copy the version string
}  /* Database::Database() */


char *Database::version()
//--------------------------------------------------
// Returns the version of the DB.  This version is a constant that is
// set automatically by RCS (in newDB.hh).  Its typicall value is
// "$ Revision: ... $".  To make it "more natural", we only return the
// string following the ":"
{
  char *v;

  for (v=_version; *v && (*v != ':'); v++) ;
  v++;
  return(v);
}


Name *Database::getName(char *s)
//--------------------------------------------------
// Find an instance of Name with name equal to s
// Creates one if it does not already exist   
{
  Name *aName;

  aName = (Name *)findInHashTable(_allNames, s);
  if(aName == NULL) {
    aName = new Name(s);
    addToHashTable(_allNames, (char *)aName);
  }
  return(aName);
}  /* Database::getName() */


Name * Database::findExistingName(char *c)
//--------------------------------------------------
// Find an object Name with name equal to s
// Return NULL if it does not already exist
{
  Name *aName;

  return (Name *)findInHashTable(_allNames, c);
}  /* Database::findExistingName() */


void Database::allNamesDo(void (func)(char *, char *), char *extraArg)
//--------------------------------------------------
{
  hashTableDo(_allNames, func, extraArg);
}  /* Database::allNamesDo() */


# ifdef DEBUG
void Database::hello()
//--------------------------------------------------
{
  int i;

  printf("version=%s\n", _version);
  for (i=0; i < MAX_CLASSES; i++) {
	printf("Class[%d] count=%d,  ", i, _counts[i]);
	if ((i % 3) == 0)
	  putchar('\n');
  }
  if (((i-1) % 3) != 0)
	putchar('\n');
  printf("List SoftObj :%d\n", _allObjects.size());
  printf("List File    :%d\n", _allFiles.size());
  printf("List Routines:%d\n", _allRoutines.size());
  printf("HashTable=0x%x, %d elements\n",
		 _allNames, numHashedInHashTable(_allNames));
}  /* Database::hello() */
# endif


// **************************************************
//					  MISCELANEOUS
// **************************************************



// get-hash-key fct used for the hashtable in the DB.
// The DB contains a pointer to this, therefore it is "sensible code"
char *dbKeyFunc(char *hashedObject)
//--------------------------------------------------
// Tim
{
  return ((Name *)hashedObject)->string();
}  /* dbKeyFunc() */
