//                              -*- Mode: C++ -*- 
// postProcessDB.cc --- perform some necessary post creation process
//                      on the DB 
// Author          : Nicolas Anquetil
// Created On      : Thu Aug 21 15:40:49 1997
// 


# include "read-API.hh"
# include "qsort.cc"


void pdate(char *msg)
//--------------------------------------------------
// prints msg + the time
{
  printf(msg);
  fflush(stdout);
  system("date +\"%H:%M:%S\"");

}


void sortNameList(char *n, char *x)
//--------------------------------------------------

{
  ((Name *)n)->sortLists();
}


void
sortLists()
// --------------------------------------------------
// sorts all the list for all object in the DB according to the order
// defined by SO_strict_cmp.
// This includes reodering the array of all objects
{
  int i,
	total, milestone;  // 2 variables to trace the progress of the sorting

  total = cdbGetObjectCount( _softwareobject);
  milestone = 10;   // first trace after 10%

  // first trace at the begining (0%)
  printf("  sorting %d objects' internal lists:", total);
  pdate(" start at ");

  for (i=0; i < total; i++) {
	// trace every 10%
	if ( (100*i/total) >= milestone) {
	  printf("  %d percent (%d objects)",milestone, i);
	  pdate(" done at: ");
	  if (milestone < 90)
		milestone += 10; // next trace 10% farther
	  else
		milestone = 999;  // unless we where at 90%, then next trace is the end
	}

	// actual sorting
	cdbGetObjectGivenKey(i)->sortLists();
  }

  pdate("  individual sorting done at: ");

  // reimplemented QuickSort to work on STL's vectors
  QuickSort(the_DB->allObjects(), 0, total-1, SO_strict_cmp);
  pdate("  all objects sorted at: ");

  the_DB->allNamesDo(sortNameList, NULL);
  pdate("  individual Names sorted at: ");
}


void
resetKeys()
// --------------------------------------------------
// after reordering the array of all objects, resets the object's key
// to their new position in the array
{
  int i;

  printf("  resetting SO keys...\n");

  for (i=0; i < cdbGetObjectCount( _softwareobject); i++)
	(*the_DB->allObjects())[i]->resetKey(i);
}


void
main()
// --------------------------------------------------
{
  int result;

  result = initGrowableFileMem( "test.mf", UFM_WRITE, 1, 10000);
  if(result != 0) { 
    printf("Failed to init file mem rc=%d\n", result);
    exit(2);
  }
  else {
    init_pointers_from_db();
  }

  sortLists();
  resetKeys();

  terminateFileMem();
}
