//                              -*- Mode: C++ -*- 
// qsort.c --- sort function for vector of the STL
//             algorithms from http://www.geocities.com/SoHo/2167/book.html
// Author          : Nicolas Anquetil
// Created On      : Mon Aug 25 10:36:40 1997
// 

# define T_ELT SoftwareObject*
# define GT(a,b) (compar((a),(b)) > 0)


void InsertSort(vector<T_ELT> *array, int lb,int ub,
				 int (*compar)(T_ELT, T_ELT))
// --------------------------------------------------
{
  T_ELT insert;
  int i, j, jmin;

   /************************
    *  sort array[lb..ub]  *
    ************************/
    jmin = lb - 1;
    for (i = lb+1; i <= ub; i++) {
        insert = (*array)[i];

        // shift elements down until insertion point found
        for (j = i-1; (j > jmin) && GT((*array)[j], insert); j--)
            (*array)[j+1] = (*array)[j];
        (*array)[j+1] = insert;
    }
}


int partition(vector<T_ELT> *array, int lb,int ub,
				 int (*compar)(T_ELT, T_ELT))
// --------------------------------------------------
{
  T_ELT exch;
  T_ELT pivot;
  int i, j, ipivot;
  unsigned int offset;

   /*****************************
    *  partition array[lb..ub]  *
    *****************************/

  // select pivot and exchange with 1st element
    offset = (ub - lb)>>1;
	ipivot = lb+offset;
    pivot = (*array)[ipivot];
	(*array)[ipivot] = (*array)[lb];

	i = lb+1;
	j = ub;
    while (true) {
        while ( (i < j)  && GT(pivot, (*array)[i]) )
		  i++;
        while ( (j >= i) && GT((*array)[j], pivot) )
		  j--;

		if (i >= j)
		  break;

        exch = (*array)[i];
        (*array)[i] = (*array)[j];
        (*array)[j] = exch;

		i++;  j--;
    }

    // pivot belongs in array[j]
    (*array)[lb] = (*array)[j];
    (*array)[j]  = pivot;

    return(j);
}


void QuickSort(vector<T_ELT> *array, int lb,int ub,
				 int (*compar)(T_ELT, T_ELT))
// --------------------------------------------------
{
  int m;

  /**************************
   *  Sort array[lb..ub]  *
   **************************/

  while (lb < ub) {

	// quickly sort short lists
	if (ub - lb <= 12) {
	  InsertSort(array, lb, ub, compar);
	  //BubbleSort(array, lb, ub, compar);
	  return;
	}

	// partition into two segments
	m = partition(array, lb, ub, compar);

	// sort the smallest partition
	// to minimize stack requirements
	if ( (m - lb) <= (ub - m) ) {
	  QuickSort(array, lb, m-1, compar);
	  lb = m + 1;
	}
	else {
	  QuickSort(array, m + 1, ub, compar);
	  ub = m - 1;
	}
  }
}
