//                              -*- Mode: C++ -*- 
// read-API.cc --- API to "Read" the new DB
// Author          : Nicolas Anquetil
// Created On      : Fri Jan 24 17:43:40 1997
// 

# include "read-API.hh"

// **************************************************
//			STATISTICS
// **************************************************

int cdbGetObjectCount(class_t filter)
//--------------------------------------------------
{
  VoidIterator<SoftwareObject*> *iter;
  int rtn;
 
  if (isCdbVirtualClass( filter)) {
	iter = cdbGetObjectList(filter);
	rtn = iter->howManyLeft();
	delete(iter);
  }
  else
	rtn = the_DB->count( filter);

  return(rtn);
}  /* cdbGetObjectCount() */


char *cdbVersion()
//--------------------------------------------------
{
  return( the_DB->version());
}  /* cdbVersion() */


// **************************************************
//			INTERNAL QUERIES
// **************************************************

SoftwareObject *cdbGetObjectGivenKey(int key)
//--------------------------------------------------
{
  return( (*the_DB->allObjects())[key]);
}  /* cdbGetObjectGivenKey() */

Iterator *cdbGetObjectListGivenKey(int key)
//--------------------------------------------------
// Same as preceding but returns an iterator
{
  SoftwareObject *obj;

  obj = cdbGetObjectGivenKey(key);

  if (obj == NULL)
	return( new VoidIterator<SoftwareObject*>());
  else
	return( new SingletonIterator<SoftwareObject*>(obj) );
}  /* cdbGetObjectListGivenKey() */


int cdbGetKeyGivenObject(SoftwareObject *object)
//--------------------------------------------------
{
  return( object->objectKey());
}  /* cdbGetKeyGivenObject() */


class_t cdbGetClassGivenObject(SoftwareObject *object)
//--------------------------------------------------
{
  return( object->classType());
}  /* cdbGetClassGivenObject() */


Iterator *cdbGetDefList(SoftwareObject *obj)
  //expects obj to be a referenceexistence, returns namesake definitions
  //strips the expected reference object at head of list.  Note: If it 
  //happens that obj was not a referenceexistence, the returned list still
  // is missing the first element.

{
  VoidIterator<SoftwareObject*> *SOiter;

  SOiter = cdbGetObjectListGivenName(obj->objectName(),_softwareobject);    
  SOiter->next();

  return(SOiter);
}


// **************************************************
//		GLOBAL QUERIES 
// **************************************************

Iterator *cdbGetObjectListGivenName(
//--------------------------------------------------
    char *name, class_t filterClass)
{
  return( getNamedObject(name, filterClass));
}  /* cdbGetObjectListGivenName() */

Iterator *cdbGetObjectListRegExp(char *regexp, class_t filterClass)
//--------------------------------------------------
{
  VoidIterator<SoftwareObject*> *SOiter;

# ifdef REGEXP
  // if the filter is SourceFile or RoutineSource, we want to go through
  // the "specialized" lists of objects which should be much faster than
  // looking to all SoftwareObjects in the DB
  if (filterClass == _sourcefile)
	SOiter = (VoidIterator<SoftwareObject*>*)
	  new LIterator<SourceFile*>(the_DB->allFiles());
  else if (filterClass == _routinesource)
	SOiter = (VoidIterator<SoftwareObject*>*)
	  new LIterator<RoutineSource*>(the_DB->allRoutines());
  else
    // to bad, we have to browse through the entire DB
	SOiter = new VIterator<SoftwareObject*>(the_DB->allObjects());

  return( new RegexpIterator<SoftwareObject*>(
	        new TypeFilterIterator<SoftwareObject*,SoftwareObject*>(
	          SOiter, filterClass),
			regexp, cdbGetObjectName));
# else
  return( new VoidIterator<SoftwareObject*>());
# endif

}  /* cdbGetObjectListRegExp() */


Iterator *cdbGetObjectList(class_t filterClass)
//--------------------------------------------------
{
  return( new TypeFilterIterator<SoftwareObject*,SoftwareObject*>(
	        new VIterator<SoftwareObject*>(
              the_DB->allObjects()),
			filterClass));
}  /* cdbGetObjectList() */


// **************************************************
//		ASSOCIATION QUERIES 
// **************************************************


Iterator *cdbGetObjectListThatDefineObject(class_t filterClass,
//--------------------------------------------------
										   SoftwareObject *object)
{
  return( object->definedBy(filterClass));
}  /* cdbGetObjectListThatDefineObject() */


Iterator *cdbGetObjectListDefinedByObject(class_t filterClass,
//--------------------------------------------------
										  SoftwareObject *object)
{
  return( object->defines(filterClass));
}  /* cdbGetObjectListDefinedByObject() */
		

Iterator *cdbGetObjectListThatReferToObject(class_t filterClass,
//--------------------------------------------------
					    SoftwareObject *object)
{
  return( object->referedBy(filterClass));
}  /* cdbGetObjectListThatReferToObject() */


Iterator *cdbGetObjectListReferedByObject(class_t filterClass,
//--------------------------------------------------
					  SoftwareObject *object)
{
  return( object->refersTo(filterClass));
}  /* cdbGetObjectListReferedByObject() */


// **************************************************
//		ATTRIBUTE QUERIES 
// **************************************************

char *cdbGetObjectName(SoftwareObject *object)
//--------------------------------------------------
{
  return(object->objectName());
}  /* cdbGetObjectName() */


void cdbGetSoftwareObjectAttributes(SoftwareObject *object,int *objectKey)
//--------------------------------------------------
{
  *objectKey = object->objectKey();
}  /* cdbGetSoftwareObjectAttributes() */


void cdbGetSourceUnitAttributes(SourceUnit *object,
//--------------------------------------------------
				Iterator **contains,
				Iterator **refersToData,
				Iterator **refersToCommentTerm,
				Iterator **defines)
{
  *contains = new CastIterator<SourceWithinFile*,SoftwareObject*>(
               new LIterator<SourceWithinFile*>( object->contains()));

  *refersToData = new CastIterator<DataUseExistence*,SoftwareObject*>(
                    new LIterator<DataUseExistence*>(
                      object->refersToData()));

  *refersToCommentTerm = new CastIterator<CommentTermExistence*,
                                          SoftwareObject*>(
                           new LIterator<CommentTermExistence*>(
                             object->refersToCommentTerm()));

  *defines = new CastIterator<StandaloneDefinition*,SoftwareObject*>(
               new LIterator<StandaloneDefinition*>( object->defines()));

}  /* cdbGetSourceUnitAttributes() */


void cdbGetSourceFileAttributes(SourceFile *object,
//---------------------------------------------------           
                                char *fileName,
				char *path,
				int *vers,
				int  *dateChanged,
				int  *size,
				Iterator **includes)
{
  strcpy(fileName, object->baseName()->string());
  strcpy(path, object->pathName());
  *vers = object->version();
  *dateChanged = object->dateChanged();
  *size = object->size();
  *includes = new CastIterator<FileInclusionExistence*,SoftwareObject*>(
                new LIterator<FileInclusionExistence*>(
                  object->includes()));
  
}  /* cdbGetSourceFileAttributes() */


void cdbGetSourceWithinFileAttributes(SourceWithinFile *object,
//--------------------------------------------------
                                      SourceUnit **containingSource,
				      int *startChar,
				      int *endChar)
{
  *containingSource = object->containingSource();
  *startChar = object->startChar();
  *endChar = object->endChar();  
}  /* cdbGetSourceWithinFileAttributes() */


void cdbGetRoutineSourceAttributes(RoutineSource *object,
//--------------------------------------------------
                                   char *Routinename,
				   bool *isEntry,
				   TypeUseExistence **returnType,
				   Iterator **calls,
				   char *defby)
{
  strcpy(Routinename,object->routineName()->string());
  *isEntry = object->isEntry();
  *returnType = object->returnType();
  *calls = new CastIterator<RoutineCallExistence*,SoftwareObject*>(
		     new LIterator<RoutineCallExistence*>(object->calls()));
  strcpy(defby,(( (object->definedBy(_softwareobject))->next())->objectName()));
  
}  /* cdbGetRoutineSourceAttributes() */


void cdbGetDefinitionAttributes(Definition *object,
//--------------------------------------------------	
			 char *definitionName)
{
  strcpy(definitionName,object->definitionName()->string());
}  /* cdbGetDefinitionAttributes() */


void cdbGetTypedDefinitionAttributes(TypedDefinition *object,
//--------------------------------------------------
	                             TypeUseExistence **type)
{
  *type = object->type();
}  /* cdbGetTypedDefinitionAttributes() */


void cdbGetStandaloneDefinitionAttributes(StandaloneDefinition *object,
//--------------------------------------------------
				int *startChar,int *endChar,
				  SourceUnit **containingSource)
{
  *startChar = object->startChar();
  *endChar = object->endChar();
  *containingSource = object->containingSource();
}  /* cdbGetStandaloneDefinitionAttributes() */		


void cdbGetFieldAttributes(Field *object,
//--------------------------------------------------
			   RecordTypeDef **isFieldMemberOf)
{
  *isFieldMemberOf = object->isFieldMemberOf();
}  /* cdbGetFieldAttributes() */


void cdbGetDatumDefAttributes(DatumDef *object,
//--------------------------------------------------
			      bool *isConst,
			      bool *isFormalArgument)
{
  *isConst = object->isConst();
  *isFormalArgument = object->isFormalArgument();  
}  /* cdbGetDatumDefAttributes() */


void cdbGetEnumerationConstAttributes(EnumerationConst *object,
//--------------------------------------------------
			     EnumeratedTypeDef **isEnumerationMemberOf)
{
  *isEnumerationMemberOf = object->isEnumerationMemberOf();
}  /* cdbGetEnumerationConstAttributes() */


void cdbGetEnumeratedTypeDefAttributes(EnumeratedTypeDef *object,
//--------------------------------------------------
 			       Iterator **enumerationMembers)
{
  *enumerationMembers = new CastIterator<EnumerationConst*,SoftwareObject*>(
                          new LIterator<EnumerationConst*>(
                            object->enumerationMembers()));
}  /* cdbGetEnumeratedTypeDefAttributes() */


void cdbGetRecordTypeDefAttributes(RecordTypeDef *object,
//--------------------------------------------------
             			   Iterator **fieldMembers)
{
  *fieldMembers = new CastIterator<Field*,SoftwareObject*>(
                    new LIterator<Field*>( object->fieldMembers()));
}  /* cdbGetRecordTypeDefAttributes() */


void cdbGetReferenceExistenceAttributes(ReferenceExistence *object,
//--------------------------------------------------
				char *referenceName)
{
  strcpy(referenceName,object->referenceName()->string());
}  /* cdbGetReferenceExistenceAttributes() */


void cdbGetCommentTermExistenceAttributes(CommentTermExistence *object,
//--------------------------------------------------
					  SourceUnit **foundInSource,
					  Iterator **successorInCompound,
					  Iterator **predecessorInCompound)
{
  *foundInSource = object->foundInSource();
  /* note: XCommentTermExistence here is needed because of a bug in g++.
     it should be CommentTermExistence */
  *successorInCompound = new CastIterator<XCommentTermExistence*,
                                          SoftwareObject*>(
                           new LIterator<XCommentTermExistence*>(
                             object->successorInCompound()));
  *predecessorInCompound = new CastIterator<XCommentTermExistence*,
                                            SoftwareObject*>(
                             new LIterator<XCommentTermExistence*>(
                               object->predecessorInCompound()));
}  /* cdbGetCommentTermExistenceAttributes() */


void cdbGetFileInclusionExistenceAttributes(FileInclusionExistence *object,
//--------------------------------------------------
				    Iterator **potentiallyIncludedIn)
{

  *potentiallyIncludedIn = new CastIterator<SourceFile*,SoftwareObject*>(
                             new LIterator<SourceFile*>(
                               object->potentiallyIncludedIn()));
}  /* cdbGetFileInclusionExistenceAttributes() */


void cdbDataUseExistenceAttributes(DataUseExistence *object,
//--------------------------------------------------
				   Iterator **foundInSource,
				   Iterator **successorInCompound,
				   Iterator **predecessorInCompound)
{
  *foundInSource = new CastIterator<SourceUnit*,SoftwareObject*>(
			         new LIterator<SourceUnit*>(
			           object->foundInSource()));
  /* note: XDataUseExistence here is needed because of a bug in g++.
     it should be DataUseExistence */
  *successorInCompound = new CastIterator<XDataUseExistence*,SoftwareObject*>(
			               new LIterator<XDataUseExistence*>(
			                 object->successorInCompound()));
  *predecessorInCompound = new CastIterator<XDataUseExistence*,
                                            SoftwareObject*>(
			                 new LIterator<XDataUseExistence*>(
			                   object->predecessorInCompound()));
}  /* cdbDataUseExistenceAttributes() */


void cdbTypeUseExistenceAttributes(TypeUseExistence *object,
//--------------------------------------------------
				   Iterator **usedAsTypeOf,
				   Iterator **usedAsReturnTypeOf)
{
  *usedAsTypeOf = new CastIterator<TypedDefinition*,SoftwareObject*>(
                    new LIterator<TypedDefinition*>(
                      object->usedAsTypeOf()));
  *usedAsReturnTypeOf = new CastIterator<RoutineSource*,SoftwareObject*>(
                          new LIterator<RoutineSource*>(
                            object->usedAsReturnTypeOf()));
}  /* cdbTypeUseExistenceAttributes() */


void cdbRoutineCallExistenceAttributes(RoutineCallExistence *object,
//--------------------------------------------------
				      Iterator **potentiallyCalledBy)
{
  *potentiallyCalledBy = new CastIterator<SourceUnit*,SoftwareObject*>(
                           new LIterator<SourceUnit *>(
						     object->potentiallyCalledBy()));
}  /* cdbRoutineCallExistenceAttributes() */ 


// **************************************************
//		ITERATORS for DataBase (on SoftwareObject)
// **************************************************

SoftwareObject *iteratorNext(Iterator *iter)
//--------------------------------------------------
{
  return( iter->next());
}  /* iteratorNext() */


bool iteratorEOI(Iterator *iter)
//--------------------------------------------------
{
  return( iter->eoi());
}  /* iteratorEOI() */


void iteratorDestroy(Iterator *iter)
//--------------------------------------------------
{
  delete( iter);
}  /* iteratorDestroy() */


int iteratorHowManyLeft(Iterator *iter)
//--------------------------------------------------
{
  return(iter->howManyLeft());
}  /* iteratorHowManyLeft() */
