
/*                               -*- Mode: C -*- 
 * test1_c.c --- test: query module
 * Author          : Sue Rao
 * Created On      : Tue Feb  4 17:51:44 1997
 */


# include "test1_c.h"
  
/* This routine returns the actual classType when passed 
   the enumerated classType */

char *className(class_t classType)
/**************************************************
 */
{
  switch( classType) {
    case _softwareobject:        return("SoftwareObject");
    case _sourceunit:            return("SourceUnit");
    case _sourcefile:            return("SourceFile");
    case _sourcewithinfile:      return("SourceWithinFile");
    case _routinesource:         return("RoutineSource");
    case _sourceblock:           return("SourceBlock");
    case _referenceexistence:    return("ReferenceExistence");
    case _commenttermexistence:  return("CommentTermExistence");
    case _specialcodeexistence:  return("SpecialCodeExistence");
    case _fileinclusionexistence:return("FileInclusionExistence");
    case _datauseexistence:      return("DataUseExistence");
    case _manifestconstexistence:return("ManifestConstExistence");
    case _typeuseexistence:      return("TypeUseExistence");
    case _routinecallexistence:  return("RoutineCallExistence");
    case _definition :           return("Definition");
    case _typeddefinition :      return("TypedDefinition"); 
    case _standalonedefinition:  return("StandAloneDefinition");
    case _field:                 return("Field");
    case _typedef:               return("TypeDef");
    case _datumdef:              return("DatumDef");
    case _enumerationconst:      return("EnumerationConst");
    case _enumeratedtypedef:     return("EnumeratedTypeDef");
    case _recordtypedef:         return("RecordTypeDef");
  }
}

/* Routine to print the pointed object's key, name & class */

void basicPrinting(SoftwareObject *object)
/**************************************************
 */
{
   printf("object with key %d has name %s and is of type %s\n",
	     cdbGetKeyGivenObject(object),
	     cdbGetObjectName(object),
	     className(cdbGetClassGivenObject(object)));
}


void objectDetails()  
/**************************************************
 */
{   
  int key;
  SoftwareObject *object;

  printf("Enter the key for the object: ");
  scanf("%d",&key);

  if ((key < 0) || (key >= cdbGetObjectCount(_softwareobject)))
    printf("key incorrect !\n");
  else {
    object = cdbGetObjectGivenKey(key);
    basicPrinting(object);
  }
  return;
}

/* prints the statistics of the loaded file */
 
void generalStatistics()
/**************************************************
 */
{
  int i;
  for (i = 0;i<= 22;i++)
  printf("No of %s = %d\n", className(i), cdbGetObjectCount(i));
}


void objectList()
/**************************************************
 */
{
  void *object;
  char objName[50];
  Iterator *iter;
 
  printf("Enter the name of the object:");
  scanf("%s", objName);

  iter = cdbGetObjectListGivenName(objName,_softwareobject);
  while (! iteratorEOI(iter)) {
    object = iteratorNext(iter);
  basicPrinting(object);
  }
}


void printAllObjectKeysAndTypes(char *iterDesc, Iterator *iter)
/**************************************************
 */
{
  SoftwareObject *object;

  printf("----------\nprinting content of iterator: %s --\n", iterDesc);
  while(! iteratorEOI(iter)) {
    object = iteratorNext(iter);
    basicPrinting(object);
  }
}

/* prints the list of objects of a particular classType */

void getObjectList()
/**************************************************
 */
{
  SoftwareObject *object;
  Iterator *iter;

  iter = cdbGetObjectList(_referenceexistence);
   while(! iteratorEOI(iter)) {
    object = iteratorNext(iter);
    basicPrinting(object);
   }
}


void SoftwareObjectAttr(SoftwareObject *object)
/**************************************************
 */
{
  int objectKey;
  
  cdbGetSoftwareObjectAttributes(object,&objectKey);
  printf("object Key %d\n", objectKey);

}


void SourceUnitAttr(SourceUnit *object)  
/**************************************************
 */
{
  Iterator *itContains,
           *itRefData,
           *itRefComment,
           *itDef;
      
  cdbGetSourceUnitAttributes(object, &itContains, &itRefData,
				 &itRefComment, &itDef);
  printAllObjectKeysAndTypes("contains", itContains);
  printAllObjectKeysAndTypes("ref data",itRefData);
  printAllObjectKeysAndTypes("ref comment",itRefComment);
  printAllObjectKeysAndTypes("def",itDef);
  iteratorDestroy(itContains);
  iteratorDestroy(itRefData);
  iteratorDestroy(itRefComment);
  iteratorDestroy(itDef);
}


void SourceFileAttr(SourceFile *object)  
/**************************************************
 */
{
  char fileName[100],
       path[100];
  int version,
      dateChanged,
      size;
  Iterator *itIncludes;

  cdbGetSourceFileAttributes(object, fileName, path, &version, &dateChanged,
			     &size, &itIncludes);
  printf("file name %s\n", fileName);
  printf("path %s\n", path);
  printf("version %d\n", version);
  printf("date changed %d\n", dateChanged);
  printf("size %d\n", size);
  printAllObjectKeysAndTypes("includes", itIncludes);
  iteratorDestroy(itIncludes);  
}


void SouceWithinFileAttr(SourceWithinFile *object)  
/**************************************************
 */
{
  SourceUnit *containingSource;
  int startChar,
      endChar;

  cdbGetSourceWithinFileAttributes(object,&containingSource,&startChar,
				       &endChar);
  basicPrinting(containingSource);
  printf("start char: %d\n",startChar);
  printf("end char: %d\n",endChar);  
}


void RoutineSourceAttr(RoutineSource *object)  
/**************************************************
 */
{
  char routineName[100];
  bool isEntry;

  TypeUseExistence *returnType;
  Iterator *itCalls;

  cdbGetRoutineSourceAttributes(object,routineName,&isEntry,&returnType,
				&itCalls);
  printf("routine name: %s\n",routineName);
  printf("is entry: %c\n",isEntry?'Y':'N');
  basicPrinting(returnType); 
  printAllObjectKeysAndTypes("calls",itCalls);
  iteratorDestroy(itCalls);
}


void DefinitionAttr(Definition *object)  
/**************************************************
 */
{
  char definitionName[100];

  cdbGetDefinitionAttributes(object,definitionName);
  printf("definition name: %s\n",definitionName);
}


void TypedDefinitionAttr(TypedDefinition *object)  
/**************************************************
 */
{
  TypeUseExistence *type;

  cdbGetTypedDefinitionAttributes(object,&type);
  basicPrinting(type);
}


void StandalonedefinitionAttr(StandaloneDefinition *object)  
/**************************************************
 */
{
  int startChar,
      endChar;
  SoftwareObject *containingSource;

  cdbGetStandaloneDefinitionAttributes(object,&startChar,&endChar,
                                         &containingSource);
  printf("start char: %d\n",startChar);
  printf("end char: %d\n",endChar);
  basicPrinting(containingSource);  
}


void FieldAttr(Field *object)  
/**************************************************
 */
{
  RecordTypeDef *isFieldMemberOf;

  cdbGetFieldAttributes(object,&isFieldMemberOf);
  basicPrinting(isFieldMemberOf);
}


void DatumDefAttr(DatumDef *object)  
/**************************************************
 */
{
  bool isConst,
       isFormalArgument;

  cdbGetDatumDefAttributes(object,&isConst,&isFormalArgument);
  printf("is const %c\n",isConst?'Y':'N');
  printf("is formal argument %c\n",isFormalArgument?'Y':'N');
}


void EnumerationConstAttr(EnumerationConst *object)  
/**************************************************
 */
{
  EnumeratedTypeDef *isEnumerationMemberOf;

  cdbGetEnumerationConstAttributes(object,&isEnumerationMemberOf);
  basicPrinting(isEnumerationMemberOf);
}


void EnumeratedTypeDefAttr(EnumeratedTypeDef *object)  
/**************************************************
 */
{
  Iterator *itenumerationMembers;

  cdbGetEnumeratedTypeDefAttributes(object,&itenumerationMembers);
  printAllObjectKeysAndTypes("enumeration members ",itenumerationMembers);
  iteratorDestroy(itenumerationMembers);
}


void RecordTypeDefAttr(RecordTypeDef *object)  
/**************************************************
 */
{
  Iterator *itfieldMembers;

  cdbGetRecordTypeDefAttributes(object,&itfieldMembers);
  printAllObjectKeysAndTypes("field members",itfieldMembers);
  iteratorDestroy(itfieldMembers);
}


void ReferenceExistenceAttr(ReferenceExistence *object)  
/**************************************************
 */
{
  char referenceName[100];
  cdbGetReferenceExistenceAttributes(object,referenceName);
  printf("reference name: %s\n",referenceName);
}


void CommentTermExistenceAttr(CommentTermExistence *object)  
/**************************************************
 */
{
  SourceUnit *fndInSource;
  Iterator *itsuccInComp,
           *itpredInComp;

  cdbGetCommentTermExistenceAttributes(object,&fndInSource,&itsuccInComp,
                                       &itpredInComp);
  basicPrinting(fndInSource);
  printAllObjectKeysAndTypes("successor in compound attributes",itsuccInComp);
  printAllObjectKeysAndTypes("predecessor in compound attributes",itpredInComp);
  iteratorDestroy(itsuccInComp);
  iteratorDestroy(itpredInComp); 
}


void FileInclusionExistenceAttr(FileInclusionExistence *object)  
/**************************************************
 */
{
  Iterator *itPotIncIn;

  cdbGetFileInclusionExistenceAttributes(object,&itPotIncIn);
  printAllObjectKeysAndTypes("potentially included attributes",itPotIncIn);
  iteratorDestroy(itPotIncIn);
}


void DataUseExistenceAttr(DataUseExistence *object)  
/**************************************************
 */
{
  Iterator *itfndInSource,
           *itsuccInComp,
           *itpredInComp;

  cdbDataUseExistenceAttributes(object,&itfndInSource,&itsuccInComp,
					  &itpredInComp);
  printAllObjectKeysAndTypes("found in source attributes",itfndInSource);
  printAllObjectKeysAndTypes("successor in compound attributes",itsuccInComp);
  printAllObjectKeysAndTypes("predeccessor in compound attributes",itpredInComp);
  iteratorDestroy(itfndInSource);
  iteratorDestroy(itsuccInComp);
  iteratorDestroy(itpredInComp);
}


void TypeUseExistenceAttr(TypeUseExistence *object)  
/**************************************************
 */
{
  Iterator *itusedAsTypeOf,
           *itusedAsReturnTypeOf;

  cdbTypeUseExistenceAttributes(object,&itusedAsTypeOf,&itusedAsReturnTypeOf);
  printAllObjectKeysAndTypes("used As Type Of",itusedAsTypeOf);
  printAllObjectKeysAndTypes("used As Return Type Of",itusedAsReturnTypeOf);
  iteratorDestroy(itusedAsTypeOf);
  iteratorDestroy(itusedAsReturnTypeOf);
}


void RoutineCallExistenceAttr(RoutineCallExistence *object)  
/**************************************************
 */
{
  Iterator *itpotentiallyCalledBy;

  cdbRoutineCallExistenceAttributes(object,&itpotentiallyCalledBy);
  printAllObjectKeysAndTypes("potentially called by",itpotentiallyCalledBy);
  iteratorDestroy(itpotentiallyCalledBy);
}

/* accepts user input and switches to appro class attributes routine */
 
void attributes()
/**************************************************
 */
{
  SoftwareObject *object;
  char  objName[100],
        *name;
  Iterator *iter;
    
  printf("Enter the name of the object:");
  scanf("%s",objName);
 
  iter = cdbGetObjectListGivenName(objName,_softwareobject);

  while (! iteratorEOI(iter)) {
	putchar('\n');
    object = iteratorNext(iter);

    if (cdbGetObjectName(object) == NULL)
      name = "noname";
    else
      name = cdbGetObjectName(object);

    basicPrinting(object);

    switch (cdbGetClassGivenObject((SoftwareObject*)object)) {

    case(_softwareobject):         SoftwareObjectAttr(object);
                                   break;
  
    case(_sourceunit):             SourceUnitAttr(object);
                                   break;

    case(_sourcefile):             SourceFileAttr(object);
                                   break;

    case(_sourcewithinfile):       SouceWithinFileAttr(object);
			           break;

    case(_routinesource):          RoutineSourceAttr(object);
                                   break;

    case(_definition):             DefinitionAttr(object);
                                   break;

    case(_typeddefinition):        TypedDefinitionAttr(object);
                                   break;

    case(_standalonedefinition):   StandalonedefinitionAttr(object);
                                   break;

    case(_field):                  FieldAttr(object); 
                                   break;

    case(_datumdef):               DatumDefAttr(object);
                                   break;

    case(_enumerationconst):       EnumerationConstAttr(object); 
                                   break;

    case(_enumeratedtypedef):      EnumeratedTypeDefAttr(object);
                                   break;

    case(_recordtypedef):          RecordTypeDefAttr(object); 
                                   break;

    case(_referenceexistence):     ReferenceExistenceAttr(object);
                                   break;

    case(_commenttermexistence):   CommentTermExistenceAttr(object); 
                                   break;

    case(_fileinclusionexistence): FileInclusionExistenceAttr(object); 
                                   break;

    case(_datauseexistence):       DataUseExistenceAttr(object);
                                   break;

    case(_typeuseexistence):       TypeUseExistenceAttr(object);
                                   break;

    case(_routinecallexistence):   RoutineCallExistenceAttr(object);
                                   break;

    default:                       printf("Object not found\n");
                                   break;
    }		              
  }
}


void objectListRefferedBy()
/**************************************************
 */
{
  SoftwareObject *object;
  int objectKey;
  char className[10];
  class_t classType;
  Iterator *iter;
    
  printf("Enter the key of the object:");
  scanf("%d", &objectKey);

  printf("Refered by what ? (r)outine, (f)ile, (v)ariable, (t)ype, (c)onst, co(m)ment\n");
  scanf("%s", className);

  switch (*className) {
  case 'r': classType = _routinesource;          break;
  case 'f': classType = _sourcefile;             break;
  case 'v': classType = _datumdef;               break;
  case 't': classType = _typedef;                break;
  case 'c': classType = _datumdef;               break;
  case 'm': classType = _commenttermexistence;   break;
  default:  classType = _definition;             break;
  }

  object = cdbGetObjectGivenKey(objectKey);
  basicPrinting(object);
  printf(" -- Referred by to : --\n");

  iter = cdbGetObjectListThatReferToObject(classType,object);
  while (! iteratorEOI(iter)) {
    object = iteratorNext(iter);
    basicPrinting(object);
  }  
}

/* check the user input command */

void checkCommand()
/**************************************************
 */
{ 
  char lineRead[100];

  *lineRead = '\0';

  while (*lineRead != 'q') {
    printf("\nEnter the command:");
    scanf("%s", lineRead);

    switch(*lineRead) {
    case 'k': objectDetails();
              break;

    case 's': generalStatistics();
              break;

    case 'n': objectList();
              break;

    case 'a': attributes();
              break;
                
    case 'r': objectListRefferedBy();
              break;

    case 'l': getObjectList();
              break;

    case 'q': break;

    default : printf("Invalid key\n");
              break;
    }
  }

  return;
}


void c_main() 
/**************************************************
 */
{ 
  printf("Enter q to quit\n");
  printf("s ---statistics\n\
k ---key no of the object\n\
n ---name of the object\n\
a ---attributes of the object\n\
r ---references\n\
l ---list of sourcefiles\n");  

  checkCommand();
}
